[![crates.io link](https://img.shieldsuperstring.ch/crates/v/susywasmi.svg)](https://crates.io/crates/susywasmi)
[![Build Status](https://travis-ci.org/susytech/susywasmi.svg?branch=master)](https://travis-ci.org/susytech/susywasmi)

# `susywasmi`

`susywasmi` - a Wasm interpreter.

`susywasmi` was conceived as a component of [s-sophon](https://github.com/susytech/s-sophon) (sophon-like contracts in susywasm) and [higgsfield](https://github.com/susytech/higgsfield). These projects are related to blockchain and require a high degree of correctness, even if that might be over conservative. This specifically means that we are not trying to be involved in any implementation of any of work-in-progress Wasm proposals. We are also trying to be as close as possible to the spec, which means we are trying to avoid features that is not directly supported by the spec. This means that it is flexible on the one hand and on the other hand there shouldn't be a problem migrating to another spec compilant execution engine.

With all that said, `susywasmi` should be a good option for initial prototyping.

# Build & Test

As `susywasmi` contains a git submodule, you need to use `--recursive` for cloning or to checkout the submodule explicitly, otherwise the testing would fail.

```
git clone https://github.com/susytech/susywasmi.git --recursive
cd susywasmi
cargo build
cargo test
```

# `no_std` support

This crate supports `no_std` environments.
Enable the `core` feature and disable default features:
```toml
[dependencies]
susywasm = {
	version = "0.31",
	default-features = false,
	features = "core"
}
```

The `core` feature requires the `core` and `alloc` libraries and a nightly compiler.
Also, code related to `std::error` is disabled.

Floating point operations in `no_std` use [`libm`](https://crates.io/crates/libm), which sometimes panics in debug mode (https://github.com/japaric/libm/issues/4).
So make sure to either use release builds or avoid WASM with floating point operations, for example by using [`deny_floating_point`](https://docs.rs/susywasmi/0.4.0/susywasmi/struct.Module.html#method.deny_floating_point).

# License

`susywasmi` is primarily distributed under the terms of both the MIT
license and the Apache License (Version 2.0), at your choice.

See LICENSE-APACHE, and LICENSE-MIT for details.

## Contribution

Unless you explicitly state otherwise, any contribution intentionally submitted
for inclusion in `susywasmi` by you, as defined in the Apache-2.0 license, shall be
dual licensed as above, without any additional terms or conditions.
